﻿------------------------
----- CREAR TAULES -----
------------------------

-----------------------------------------------------------------
--- Crearem totes les taules de la BDs, cadascuna al seu --------
--- tablespace. El propietari de totes les taules es l-usuari ---
--- administrador. ----------------------------------------------
-----------------------------------------------------------------
--- Tambe crearen les sequencies que farem servir per els camps -----------------
--- identificador de cada taula. Aquest identificador el formarem ---------------
--- posant un prefixe que identifique la taula. Per exemple l-identificador -----
--- de la taula usuari sera 'USU1000001'. Hem decidit comencar la numeracio -----
--- de cada sequencia per 10000001 per evitar operacions per omplir -------------
--- el camp amb ceros i aixi que totes les claus tinguen la mateixa -------------
--- logitud. --------------------------------------------------------------------
---------------------------------------------------------------------------------
--- l-ordre de creacio de les taules es important perque quan definim una -------
--- clau forana la taula a la que "apunta" ha d-existir. ------------------------
---------------------------------------------------------------------------------
-- tots els usuaris tindran permisos de INSERT, SELECT i UPDATE sobre totes -----
-- les taules. El permis DELETE no el necessiten. No definim permisos concrets --
-- per a cada taula ja que cada funcio comprovara quin usuari esta executant-la -
---------------------------------------------------------------------------------

------------------------------
----- TABLESPACE GESTION -----
------------------------------

--- A aquest tablespace tindrem les taules principals de la logica de negoci ---

----------------------------------------------------------
CREATE TABLE tipus_dispositiu
(
  codi_tipus character(10) NOT NULL,
  descripcio_tipus character(30) NOT NULL,
  CONSTRAINT pk_tipus_dispositiu PRIMARY KEY (codi_tipus)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE tipus_dispositiu
  OWNER TO administrador;
GRANT ALL
  ON TABLE tipus_dispositiu TO PUBLIC;
---------------------------------------------------------
CREATE TABLE dispositiu
(
  codi_dispositiu character(10) NOT NULL,
  codi_tipus character(10) NOT NULL,
  numero_serie character(20) NOT NULL,
  "data_instal_lacio" date NOT NULL,
  avariat boolean,
  valor real,
  CONSTRAINT pk_dispositiu PRIMARY KEY (codi_dispositiu),
  CONSTRAINT fk_tipus_dispositiu FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE dispositiu
  OWNER TO administrador;
GRANT ALL
  ON TABLE dispositiu TO PUBLIC;

CREATE INDEX fki_tipus_dispositiu
  ON dispositiu
  USING btree
  (codi_tipus COLLATE pg_catalog."default"); 
-------------------------------------------------------------
CREATE TABLE avaries
(
  id_avaria character(10) NOT NULL,
  codi_dispositiu character(10) NOT NULL,
  data_avaria date NOT NULL,
  descripcio_avaria text,
  data_reparacio date,
  CONSTRAINT pk_id_avaria PRIMARY KEY (id_avaria),
  CONSTRAINT fk_dispositiu FOREIGN KEY (codi_dispositiu)
      REFERENCES dispositiu (codi_dispositiu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);
ALTER TABLE avaries
  OWNER TO administrador;
GRANT ALL
  ON TABLE avaries TO PUBLIC;

CREATE INDEX fki_dispositiu
  ON avaries
  USING btree
  (codi_dispositiu COLLATE pg_catalog."default");
-----------------------------------------------------------------
CREATE TABLE pla_inversions
(
  codi_pla character(10) NOT NULL,
  "any" integer NOT NULL,
  codi_tipus character(10) NOT NULL,
  quantitat integer NOT NULL,
  import real NOT NULL,
  CONSTRAINT pk_pla_inversions PRIMARY KEY (codi_pla, "any"),
  CONSTRAINT fk_tipus_dispositiu4 FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT unique_any_codi_tipus UNIQUE ("any", codi_tipus)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE pla_inversions
  OWNER TO administrador;
GRANT ALL
  ON TABLE pla_inversions TO PUBLIC;

CREATE INDEX fki_tipus_dispositiu4
  ON pla_inversions
  USING btree
  (codi_tipus COLLATE pg_catalog."default");
--------------------------------------------------------------------
CREATE TABLE amortitzacio
(
  id_amortitzacio character(10) NOT NULL,
  codi_tipus character(10) NOT NULL,
  valor_comptable real NOT NULL,
  data_recepcio date NOT NULL,
  CONSTRAINT pk_amortitzacio PRIMARY KEY (id_amortitzacio),
  CONSTRAINT pk_codi_tipus FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE amortitzacio
  OWNER TO administrador;
GRANT ALL ON TABLE amortitzacio TO public;

CREATE INDEX fki_pk_codi_tipus
  ON amortitzacio
  USING btree
  (codi_tipus COLLATE pg_catalog."default");
-------------------------------------------------------------
CREATE TABLE proveidor
(
  codi_proveidor character(10) NOT NULL,
  "NIF" character(9) NOT NULL,
  nom character(30) NOT NULL,
  adreca character(30),
  ciutat character(30),
  provincia character(30),
  codi_postal character(5),
  baixa boolean NOT NULL DEFAULT false,
  CONSTRAINT pk_proveidor PRIMARY KEY (codi_proveidor)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE proveidor
  OWNER TO administrador;
GRANT ALL
  ON TABLE proveidor TO PUBLIC;
--------------------------------------------------------------
CREATE TABLE comandes
(
  codi_comanda character(10) NOT NULL,
  codi_proveidor character(10) NOT NULL,
  data date NOT NULL,
  ordre_inversio boolean NOT NULL DEFAULT false,
  CONSTRAINT pk_comandes PRIMARY KEY (codi_comanda),
  CONSTRAINT fk_proveidor2 FOREIGN KEY (codi_proveidor)
      REFERENCES proveidor (codi_proveidor) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE comandes
  OWNER TO administrador;
GRANT ALL
  ON TABLE comandes TO PUBLIC;

CREATE INDEX fki_proveidor2
  ON comandes
  USING btree
  (codi_proveidor COLLATE pg_catalog."default");
------------------------------------------------------------------
CREATE TABLE linies_comanda
(
  codi_comanda character(10) NOT NULL,
  numero_linia integer NOT NULL,
  codi_tipus character(10) NOT NULL,
  unitats integer NOT NULL,
  preu_unitari real NOT NULL,
  CONSTRAINT pk_linies_comanda PRIMARY KEY (codi_comanda, numero_linia),
  CONSTRAINT fk_comanda FOREIGN KEY (codi_comanda)
      REFERENCES comandes (codi_comanda) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE,
  CONSTRAINT fk_tipus_dispositiu3 FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE linies_comanda
  OWNER TO administrador;
GRANT ALL
  ON TABLE linies_comanda TO PUBLIC;

CREATE INDEX fki_tipus_dispositiu3
  ON linies_comanda
  USING btree
  (codi_tipus COLLATE pg_catalog."default");
----------------------------------------------------------------
CREATE TABLE temps_lliurament
(
  codi_proveidor character(10) NOT NULL,
  codi_comanda character(10) NOT NULL,
  temps_lliurament integer NOT NULL,
  CONSTRAINT pk_temps_lliurament PRIMARY KEY (codi_proveidor, codi_comanda),
  CONSTRAINT fk_comanda FOREIGN KEY (codi_comanda)
      REFERENCES comandes (codi_comanda) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_proveidor FOREIGN KEY (codi_proveidor)
      REFERENCES proveidor (codi_proveidor) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE temps_lliurament
  OWNER TO administrador;
GRANT ALL
  ON TABLE temps_lliurament TO PUBLIC;

CREATE INDEX fki_comanda
  ON temps_lliurament
  USING btree
  (codi_comanda COLLATE pg_catalog."default");

CREATE INDEX fki_proveidor3
  ON temps_lliurament
  USING btree
  (codi_proveidor COLLATE pg_catalog."default");
--------------------------------------------------------------------
CREATE TABLE pressupostos
(
  codi_pressupost character(10) NOT NULL,
  codi_proveidor character(10) NOT NULL,
  data date NOT NULL,
  CONSTRAINT pk_pressupostos PRIMARY KEY (codi_pressupost),
  CONSTRAINT fk_proveidor FOREIGN KEY (codi_proveidor)
      REFERENCES proveidor (codi_proveidor) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE pressupostos
  OWNER TO administrador;
GRANT ALL
  ON TABLE pressupostos TO PUBLIC;

CREATE INDEX fki_proveidor
  ON pressupostos
  USING btree
  (codi_proveidor COLLATE pg_catalog."default");
----------------------------------------------------------------
CREATE TABLE linies_pressupost
(
  codi_pressupost character(10) NOT NULL,
  numero_linia integer NOT NULL,
  codi_tipus character(10) NOT NULL,
  preu real DEFAULT 0.0,
  CONSTRAINT pk_linies_pressupost PRIMARY KEY (codi_pressupost, numero_linia),
  CONSTRAINT fk_pressupostos FOREIGN KEY (codi_pressupost)
      REFERENCES pressupostos (codi_pressupost) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE,
  CONSTRAINT fk_tipus_dispositiu2 FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE linies_pressupost
  OWNER TO administrador;
GRANT ALL
  ON TABLE linies_pressupost TO PUBLIC;

CREATE INDEX fki_tipus_dispositiu2
  ON linies_pressupost
  USING btree
  (codi_tipus COLLATE pg_catalog."default");
---------------------------------------------------------------
CREATE TABLE seu
(
  codi_seu character(10) NOT NULL,
  poblacio character(30) NOT NULL,
  provincia character(30) NOT NULL,
  adreca character(30),
  CONSTRAINT pk_seu PRIMARY KEY (codi_seu)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE seu
  OWNER TO administrador;
GRANT ALL
  ON TABLE seu TO PUBLIC;

------------------------------------------------------------------------
--- La clau forana de departament respecte de usuari per identificar ---
--- al responsable del departament la definirem despres de crear -------
--- la taula usuari ----------------------------------------------------
------------------------------------------------------------------------

CREATE TABLE departament
(
  codi_departament character(10) NOT NULL,
  nom_departament character(30) NOT NULL,
  codi_responsable character(10) DEFAULT NULL,
  CONSTRAINT pk_departament PRIMARY KEY (codi_departament)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE departament
  OWNER TO administrador;
GRANT ALL
  ON TABLE departament TO PUBLIC;
----------------------------------------------------------------
CREATE TABLE inventari
(
  codi_inventari character(10) NOT NULL,
  data date NOT NULL,
  codi_departament character(10),
  CONSTRAINT pk_inventari PRIMARY KEY (codi_inventari),
  CONSTRAINT fk_departament2 FOREIGN KEY (codi_departament)
      REFERENCES departament (codi_departament) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE inventari
  OWNER TO administrador;
GRANT ALL
  ON TABLE inventari TO PUBLIC;

CREATE INDEX fki_departament2
  ON inventari
  USING btree
  (codi_departament COLLATE pg_catalog."default");
-----------------------------------------------------------
CREATE TABLE linies_inventari
(
  codi_inventari character(10) NOT NULL,
  codi_dispositiu character(10) NOT NULL,
  CONSTRAINT pk_linies_inventari PRIMARY KEY (codi_inventari, codi_dispositiu),
  CONSTRAINT fk_dispositiu2 FOREIGN KEY (codi_dispositiu)
      REFERENCES dispositiu (codi_dispositiu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE,
  CONSTRAINT fk_inventari FOREIGN KEY (codi_inventari)
      REFERENCES inventari (codi_inventari) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);
ALTER TABLE linies_inventari
  OWNER TO administrador;
GRANT ALL ON TABLE linies_inventari TO public;

CREATE INDEX fki_dispositiu2
  ON linies_inventari
  USING btree
  (codi_dispositiu COLLATE pg_catalog."default");
------------------------------------------------------------------------------
CREATE TABLE departament_seu
(
  codi_departament character(10) NOT NULL,
  codi_seu character(10) NOT NULL,
  CONSTRAINT pk_departament_seu PRIMARY KEY (codi_departament, codi_seu),
  CONSTRAINT fk_departament FOREIGN KEY (codi_departament)
      REFERENCES departament (codi_departament) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_seu2 FOREIGN KEY (codi_seu)
      REFERENCES seu (codi_seu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE departament_seu
  OWNER TO administrador;
GRANT ALL
  ON TABLE departament_seu TO PUBLIC;

CREATE INDEX fki_seu2
  ON departament_seu
  USING btree
  (codi_seu COLLATE pg_catalog."default");
------------------------------------------------------
CREATE TABLE categoria
(
  codi_categoria character(10) NOT NULL,
  nom_categoria character(30) NOT NULL,
  CONSTRAINT pk_categoria PRIMARY KEY (codi_categoria)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE categoria
  OWNER TO administrador;
GRANT ALL
  ON TABLE categoria TO PUBLIC;
--------------------------------------------------------------------
CREATE TABLE usuari
(
  codi_usuari character(10) NOT NULL,
  nom character(50) NOT NULL,
  codi_departament character(10) NOT NULL,
  codi_seu character(10) NOT NULL,
  codi_categoria character(10) NOT NULL,
  telefon character(9),
  CONSTRAINT pk_usuari PRIMARY KEY (codi_usuari),
  CONSTRAINT fk_categoria FOREIGN KEY (codi_categoria)
      REFERENCES categoria (codi_categoria) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_departament_seu FOREIGN KEY (codi_departament, codi_seu)
      REFERENCES departament_seu (codi_departament, codi_seu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE usuari
  OWNER TO administrador;
GRANT ALL
  ON TABLE usuari TO PUBLIC;

CREATE INDEX fki_categoria
  ON usuari
  USING btree
  (codi_categoria COLLATE pg_catalog."default");

CREATE INDEX fki_departament_seu
  ON usuari
  USING btree
  (codi_departament COLLATE pg_catalog."default", codi_seu COLLATE pg_catalog."default");

----------------------------------------------------------------------------
---- Ara podrem crear la clau forana de Departament per al responsable -----
----------------------------------------------------------------------------

ALTER TABLE departament

  ADD CONSTRAINT fk_responsable FOREIGN KEY (codi_responsable)

      REFERENCES usuari (codi_usuari) MATCH SIMPLE

      ON UPDATE NO ACTION ON DELETE SET NULL;

CREATE INDEX fki_responsable

  ON departament
  USING btree

  (codi_responsable
 COLLATE pg_catalog."default");
-------------------------------------------------------------------------
CREATE TABLE peticio_material
(
  codi_peticio character(10) NOT NULL,
  codi_usuari character(10) NOT NULL,
  codi_tipus character(10) NOT NULL,
  quantitat integer NOT NULL,
  estat character(1) NOT NULL,
  data_peticio date NOT NULL,
  CONSTRAINT pk_peticio_material PRIMARY KEY (codi_peticio),
  CONSTRAINT fk_tipus_dispositiu5 FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_usuari FOREIGN KEY (codi_usuari)
      REFERENCES usuari (codi_usuari) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);
ALTER TABLE peticio_material
  OWNER TO administrador;
GRANT ALL
  ON TABLE peticio_material TO PUBLIC;

CREATE INDEX fki_tipus_dispositiu5
  ON peticio_material
  USING btree
  (codi_tipus COLLATE pg_catalog."default");

CREATE INDEX fki_usuari
  ON peticio_material
  USING btree
  (codi_usuari COLLATE pg_catalog."default");
-------------------------------------------------------------------
CREATE TABLE recepcio_material
(
  id_recepcio character(10) NOT NULL,
  codi_comanda character(10) NOT NULL,
  numero_linia integer NOT NULL,
  numero_serie character(20) NOT NULL,
  data_recepcio date NOT NULL,
  temps_lliurament integer NOT NULL,
  CONSTRAINT pk_recepcio PRIMARY KEY (id_recepcio),
  CONSTRAINT fk_linies_comanda FOREIGN KEY (codi_comanda, numero_linia)
      REFERENCES linies_comanda (codi_comanda, numero_linia) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE recepcio_material
  OWNER TO administrador;
GRANT ALL
  ON TABLE recepcio_material TO PUBLIC;
-------------------------------------------------------------------
CREATE TABLE moviments
(
  codi_moviment character(10) NOT NULL,
  tipus character(30),
  codi_dispositiu character(10) NOT NULL,
  codi_usuari_origen character(10),
  codi_usuari_desti character(10),
  data date NOT NULL,
  CONSTRAINT pk_moviments PRIMARY KEY (codi_moviment),
  CONSTRAINT fk_dispositiu4 FOREIGN KEY (codi_dispositiu)
      REFERENCES dispositiu (codi_dispositiu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT fk_usuari_desti FOREIGN KEY (codi_usuari_desti)
      REFERENCES usuari (codi_usuari) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT fk_usuari_origen FOREIGN KEY (codi_usuari_origen)
      REFERENCES usuari (codi_usuari) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL
)
WITH (
  OIDS=FALSE
);
ALTER TABLE moviments
  OWNER TO administrador;
GRANT ALL
  ON TABLE moviments TO PUBLIC;

CREATE INDEX fki_dispositiu4
  ON moviments
  USING btree
  (codi_dispositiu COLLATE pg_catalog."default");

CREATE INDEX fki_usuari_desti
  ON moviments
  USING btree
  (codi_usuari_desti COLLATE pg_catalog."default");

CREATE INDEX fki_usuari_origen
  ON moviments
  USING btree
  (codi_usuari_origen COLLATE pg_catalog."default");
-------------------------------------------------------------------------
CREATE TABLE assignat
(
  codi_usuari character(10) NOT NULL,
  codi_dispositiu character(10) NOT NULL,
  data date NOT NULL,
  CONSTRAINT pk_assignat PRIMARY KEY (codi_usuari, codi_dispositiu),
  CONSTRAINT fk_dispositiu3 FOREIGN KEY (codi_dispositiu)
      REFERENCES dispositiu (codi_dispositiu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE,
  CONSTRAINT fk_usuari FOREIGN KEY (codi_usuari)
      REFERENCES usuari (codi_usuari) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);
ALTER TABLE assignat
  OWNER TO administrador;
GRANT ALL
  ON TABLE assignat TO PUBLIC;

CREATE INDEX fki_dispositiu3
  ON assignat
  USING btree
  (codi_dispositiu COLLATE pg_catalog."default");
----------------------------------------------------------------------

------------------------------------
----- TABLESPACE ESTADISTIQUES -----
------------------------------------
-----------------------------------------------------------------------------------
-- a les taules del paquet estadistic tindran tots els permisos tots els usuaris --
-- ja que s-actualitzen des de moltes funcions diferents i per diversos -----------
-- usuaris ------------------------------------------------------------------------
-----------------------------------------------------------------------------------

-----------------------------------------------------------
CREATE TABLE "EST1"
(
  "any" integer NOT NULL,
  nombre_disps integer NOT NULL,
  CONSTRAINT "pk_EST1" PRIMARY KEY ("any")
  USING INDEX TABLESPACE estadistiques
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST1"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST1" TO PUBLIC;
-----------------------------------------------------------
CREATE TABLE "EST2"
(
  percentatge real
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST2"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST2" TO PUBLIC;
-------------------------------------------------------------
CREATE TABLE "EST3"
(
  "any" integer NOT NULL,
  import real NOT NULL,
  CONSTRAINT "pk_EST3" PRIMARY KEY ("any")
  USING INDEX TABLESPACE estadistiques
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST3"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST3" TO PUBLIC;
---------------------------------------------------------------------
CREATE TABLE "EST4"
(
  codi_proveidor character(10) NOT NULL,
  CONSTRAINT fk_proveidor4 FOREIGN KEY (codi_proveidor)
      REFERENCES proveidor (codi_proveidor) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST4"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST4" TO PUBLIC;

CREATE INDEX fki_proveidor4
  ON "EST4"
  USING btree
  (codi_proveidor COLLATE pg_catalog."default");
--------------------------------------------------------
CREATE TABLE "EST5"
(
  codi_departament character(10) NOT NULL,
  codi_tipus character(10) NOT NULL,
  nombre_disp integer NOT NULL,
  CONSTRAINT "pk_EST5" PRIMARY KEY (codi_departament, codi_tipus)
  USING INDEX TABLESPACE estadistiques,
  CONSTRAINT fk_departament FOREIGN KEY (codi_departament)
      REFERENCES departament (codi_departament) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_tipus_dispositiu6 FOREIGN KEY (codi_tipus)
      REFERENCES tipus_dispositiu (codi_tipus) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST5"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST5" TO PUBLIC;

CREATE INDEX fki_tipus_dispositiu6
  ON "EST5"
  USING btree
  (codi_tipus COLLATE pg_catalog."default");
----------------------------------------------------------------------
CREATE TABLE "EST6"
(
  codi_departament character(10) NOT NULL,
  valor real NOT NULL,
  CONSTRAINT "pk_EST6" PRIMARY KEY (codi_departament)
  USING INDEX TABLESPACE estadistiques,
  CONSTRAINT fk_departament FOREIGN KEY (codi_departament)
      REFERENCES departament (codi_departament) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST6"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST6" TO PUBLIC;
--------------------------------------------------------------------
CREATE TABLE "EST8"
(
  "any" integer NOT NULL,
  codi_departament character(10) NOT NULL,
  CONSTRAINT pk_est8 PRIMARY KEY ("any", codi_departament),
  CONSTRAINT fk_departament FOREIGN KEY (codi_departament)
      REFERENCES departament (codi_departament) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST8"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST8" TO PUBLIC;

CREATE INDEX fki_departament
  ON "EST8"
  USING btree
  (codi_departament COLLATE pg_catalog."default");
------------------------------------------------------------------
CREATE TABLE "EST9"
(
  percentatge real NOT NULL
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST9"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST9" TO PUBLIC;
------------------------------------------------------------------------
CREATE TABLE "EST10"
(
  codi_tipus character(10) NOT NULL,
  unitats integer NOT NULL,
  CONSTRAINT "pk_EST10" PRIMARY KEY (codi_tipus)
  USING INDEX TABLESPACE estadistiques
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST10"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST10" TO PUBLIC;
-------------------------------------------------------------
CREATE TABLE "EST11"
(
  codi_departament character(10) NOT NULL,
  quantitat integer NOT NULL,
  CONSTRAINT "pk_EST11" PRIMARY KEY (codi_departament)
  USING INDEX TABLESPACE estadistiques,
  CONSTRAINT fk_departament FOREIGN KEY (codi_departament)
      REFERENCES departament (codi_departament) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST11"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST11" TO PUBLIC;
-----------------------------------------------------------------
CREATE TABLE "EST12"
(
  quant_users integer NOT NULL
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST12"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST12" TO PUBLIC;
---------------------------------------------------------------------
CREATE TABLE "EST13"
(
  "any" integer NOT NULL,
  nombre_avaries integer NOT NULL,
  CONSTRAINT "pk_EST13" PRIMARY KEY ("any")
  USING INDEX TABLESPACE estadistiques
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST13"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST13" TO PUBLIC;
------------------------------------------------------------------------
CREATE TABLE "EST14"
(
  codi_usuari character(10) NOT NULL,
  CONSTRAINT "pk_EST14" PRIMARY KEY (codi_usuari)
  USING INDEX TABLESPACE estadistiques,
  CONSTRAINT fk_usuari FOREIGN KEY (codi_usuari)
      REFERENCES usuari (codi_usuari) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
TABLESPACE estadistiques;
ALTER TABLE "EST14"
  OWNER TO administrador;
GRANT ALL
  ON TABLE "EST14" TO PUBLIC;
-----------------------------------------------------------------------

---------------------------
----- TABLESPACE LOGS -----
---------------------------
-------------------------------------------------------------------------------
-- sobre la taula logs tindran unicament el permis d-INSERT tots els usuaris --
-------------------------------------------------------------------------------

-----------------------------------------------

CREATE TABLE logs
(
  id_log character(10) NOT NULL,
  data_exec date NOT NULL,
  usuari character(30) NOT NULL,
  taula character(30) NOT NULL,
  procediment character(20) NOT NULL,
  params_entrada character(300),
  params_exida character(100) NOT NULL,
  CONSTRAINT pk_logs PRIMARY KEY (id_log)
  USING INDEX TABLESPACE logs
)
WITH (
  OIDS=FALSE
)
TABLESPACE logs;

ALTER TABLE logs
  OWNER TO administrador;
GRANT INSERT
  ON TABLE logs TO PUBLIC;

---------------------------
----- CREAR SEQUENCES -----
---------------------------
--------------------------------------------------------
-- el propietari de les sequencies sera administrador --
-- tots els usuaris tindran permisos totals sobre ------
-- les sequencies --------------------------------------
--------------------------------------------------------
CREATE SEQUENCE seq_dis
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_dis
  OWNER TO administrador;
GRANT ALL
  ON seq_dis TO PUBLIC;
-------------------------------
CREATE SEQUENCE seq_ava
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_ava
  OWNER TO administrador;
GRANT ALL
  ON seq_ava TO PUBLIC;
--------------------------------
CREATE SEQUENCE seq_cat
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_cat
  OWNER TO administrador;
GRANT ALL
  ON seq_cat TO PUBLIC;
-----------------------------------
CREATE SEQUENCE seq_com
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_com
  OWNER TO administrador;
GRANT ALL
  ON seq_com TO PUBLIC;
-------------------------------------
CREATE SEQUENCE seq_dep
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_dep
  OWNER TO administrador;
GRANT ALL
  ON seq_dep TO PUBLIC;
-------------------------------------
CREATE SEQUENCE seq_inv
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_inv
  OWNER TO administrador;
GRANT ALL
  ON seq_inv TO PUBLIC;
-------------------------------------
CREATE SEQUENCE seq_mov
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_mov
  OWNER TO administrador;
GRANT ALL
  ON seq_mov TO PUBLIC;
---------------------------------------
CREATE SEQUENCE seq_pet
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_pet
  OWNER TO administrador;
GRANT ALL
  ON seq_pet TO PUBLIC;
-----------------------------------------
CREATE SEQUENCE seq_pla
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_pla
  OWNER TO administrador;
GRANT ALL
  ON seq_pla TO PUBLIC;
-----------------------------------------
CREATE SEQUENCE seq_pre
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_pre
  OWNER TO administrador;
GRANT ALL
  ON seq_pre TO PUBLIC;
------------------------------------------
CREATE SEQUENCE seq_pro
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_pro
  OWNER TO administrador;
GRANT ALL
  ON seq_pro TO PUBLIC;
----------------------------------------
CREATE SEQUENCE seq_seu
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_seu
  OWNER TO administrador;
GRANT ALL
  ON seq_seu TO PUBLIC;
--------------------------------------
CREATE SEQUENCE seq_tip
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_tip
  OWNER TO administrador;
GRANT ALL
  ON seq_tip TO PUBLIC;
---------------------------------
CREATE SEQUENCE seq_usu
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_usu
  OWNER TO administrador;
GRANT ALL
  ON seq_usu TO PUBLIC;
----------------------------------
CREATE SEQUENCE seq_log
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_log
  OWNER TO administrador;
GRANT ALL
  ON seq_log TO PUBLIC;
------------------------------------
CREATE SEQUENCE seq_rec
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_rec
  OWNER TO administrador;
GRANT ALL
  ON seq_rec TO PUBLIC;
-----------------------------------
CREATE SEQUENCE seq_amo
  INCREMENT 1
  MINVALUE 1000000
  MAXVALUE 9999999
  START 1000001
  CACHE 1;
ALTER TABLE seq_amo
  OWNER TO administrador;
GRANT ALL
  ON seq_amo TO PUBLIC;
-------------------------------------