﻿-------------------------------------
--- actuacions sobre la taula seu ---
---------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_seu (p_poblacio character(30), p_provincia character(30),
	p_adreca character(30)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO seu (codi_seu, poblacio, provincia, adreca)
    VALUES ('SEU' || nextval('seq_seu'), upper(p_poblacio), upper(p_provincia), upper(p_adreca));

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'INSERT', p_poblacio || '; ' || p_provincia || '; ' || p_adreca, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'INSERT', p_poblacio || '; ' || p_provincia || '; ' || p_adreca, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'INSERT', p_poblacio || '; ' || p_provincia || '; ' || p_adreca, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_seu (p_codi_seu character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si el registre existeix
  SELECT count(1) INTO comptador FROM seu
    WHERE codi_seu = p_codi_seu;
  IF comptador = 0 THEN
    RAISE EXCEPTION no_data_found;
  END IF;

-------------------------------------------------------------------------------------------------
-- esborrem el registre
-- Quan esborrem un registre a la taula seu, s'esborraran automaticament els registres associats
-- de les taules inventari i linies_inventari per la constraint DELETE CASCADE d'aquestes
-------------------------------------------------------------------------------------------------
  DELETE FROM seu
    WHERE codi_seu = p_codi_seu;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'DELETE', p_codi_seu, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'DELETE', p_codi_seu, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, el codi de seu no existeix a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'DELETE', p_codi_seu, RSP);
      RETURN RSP; 

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'DELETE', p_codi_seu, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_seu (p_codi_seu character(10), p_poblacio character(30), 
  p_provincia character(30), p_adreca character(30)) RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si existeix el registre a modificar
  SELECT count(1) INTO comptador FROM seu
    WHERE codi_seu = p_codi_seu;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- actualitzem els camps, els parametres dels quals no estan en blanc
  IF p_poblacio <> '' THEN
    UPDATE seu SET poblacio = upper(p_poblacio)
    WHERE codi_seu = p_codi_seu;
  END IF;

  IF p_provincia <> '' THEN
    UPDATE seu SET provincia = upper(p_provincia)
    WHERE codi_seu = p_codi_seu;
  END IF;

  IF p_adreca <> '' THEN
    UPDATE seu SET adreca = upper(p_adreca)
    WHERE codi_seu = p_codi_seu;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'UPDATE', p_codi_seu || '; ' || 
           p_poblacio || '; ' || p_provincia || ';' || p_adreca, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'UPDATE', p_codi_seu || '; ' || 
               p_poblacio || '; ' || p_provincia || ';' || p_adreca, RSP);
  RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, codi de seu no existeix a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'UPDATE', p_codi_seu || '; ' || 
               p_poblacio || '; ' || p_provincia || ';' || p_adreca, RSP);
  RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'seu', 'UPDATE', p_codi_seu || '; ' || 
               p_poblacio || '; ' || p_provincia || ';' || p_adreca, RSP);
  RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;




-------------------------------------------------------------------------------------------------------
--- actuacions sobre la taula departament ---
-------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_departament (p_nom_departament character(30))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO departament (codi_departament, nom_departament)
    VALUES ('DEP' || nextval('seq_dep'), upper(p_nom_departament));
  
-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'INSERT', p_nom_departament, RSP);

  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'INSERT', p_nom_departament, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'INSERT', p_nom_departament, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_departament (p_codi_departament character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);
  error_fk character(1) := '';

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que el departament existeix
  SELECT count(1) INTO comptador FROM departament
    WHERE codi_departament = p_codi_departament;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-------------------------------------------------------------------------------------
-- esborrem el registre
-- si l-usuari esborrat fora responsable de un departament aquest camp es faria NULL
-- per la restriccio ON DELETE SET NULL de la clau forana de departament respecte
-- d'usuari
-------------------------------------------------------------------------------------
  DELETE FROM departament
    WHERE codi_departament = p_codi_departament;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'DELETE', p_codi_departament, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'DELETE', p_codi_departament, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix un departament amb eixe codi';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'DELETE', p_codi_departament, RSP);
      RETURN RSP;

-- departament amb usuaris o existeix a la taula departament-seu
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, el departament encara te usuaris assignats o be apereix a la taula departament_seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'DELETE', p_codi_departament, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'DELETE', p_codi_departament, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_departament (p_codi_departament character(10), p_nom_departament character(30), 
  p_codi_responsable character(10)) RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si el departament existeix
  SELECT count(1) INTO comptador FROM departament
    WHERE codi_departament = p_codi_departament;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- modifiquem els camps, els parametres dels quals no son buits
  IF p_nom_departament <> '' THEN
    UPDATE departament
      SET nom_departament = p_nom_departament
      WHERE codi_departament = p_codi_departament;
  END IF;

--------------------------------------------------------------------------------
-- si anem a modificar el codi del responsable, comprobarem abans que l-usuari
-- existeix i que pertany al departament
--------------------------------------------------------------------------------
  IF p_codi_responsable <> '' THEN
    IF p_codi_departament IN (
                             SELECT codi_departament
                             FROM usuari
                             WHERE codi_usuari = p_codi_responsable
                             ) THEN
      UPDATE departament
        SET codi_responsable = p_codi_responsable
        WHERE codi_departament = p_codi_departament;
    ELSE
      RAISE ambiguous_parameter;
    END IF;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'UPDATE', p_codi_departament ||
           '; ' || p_nom_departament || '; ' || p_codi_responsable, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari no te permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'UPDATE', p_codi_departament ||
           '; ' || p_nom_departament || '; ' || p_codi_responsable, RSP);
      RETURN RSP;

-- el departament no existeix
    WHEN no_data_found THEN
      RSP = 'ERROR, codi de departament no existeix a la taula departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'UPDATE', p_codi_departament ||
           '; ' || p_nom_departament || '; ' || p_codi_responsable, RSP);
      RETURN RSP;

-- l-usuari que el vol posar com a responsable no existeix
    WHEN ambiguous_parameter THEN
      RSP = 'ERROR, l-usuari no pertany al departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'UPDATE', p_codi_departament ||
           '; ' || p_nom_departament || '; ' || p_codi_responsable, RSP);
      RETURN RSP;

-- l-usuari que el vol posar com a responsable no existeix
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, usuari inexistent';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'UPDATE', p_codi_departament ||
           '; ' || p_nom_departament || '; ' || p_codi_responsable, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio a la taula departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament', 'UPDATE', p_codi_departament ||
           '; ' || p_nom_departament || '; ' || p_codi_responsable, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;





-------------------------------------------------------------------------------------------------------
--- actuacions sobre la taula departament-seu ---
-------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_departament_seu (p_codi_departament character(10), p_codi_seu character(10))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer := 0;
  error_fk character(1) := '';

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO departament_seu (codi_departament, codi_seu)
    VALUES (p_codi_departament, p_codi_seu);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'INSERT', p_codi_departament || '; ' ||
           p_codi_seu, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'INSERT', p_codi_departament || '; ' ||
           p_codi_seu, RSP);
      RETURN RSP;

-- ja existeix un registre
    WHEN unique_violation THEN
      RSP = 'ERROR, ja existeix un registre amb el codi seu i codi departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'INSERT', p_codi_departament || '; ' ||
           p_codi_seu, RSP);
      RETURN RSP;

-- no existeix la seu o el departament
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, no existeix el departament o la seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'INSERT', p_codi_departament || '; ' ||
           p_codi_seu, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'INSERT', p_codi_departament || '; ' ||
           p_codi_seu, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_departament_seu (p_codi_departament character(10), p_codi_seu character(10))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que el registre existeix
  SELECT count(1) INTO comptador FROM departament_seu
    WHERE codi_departament = p_codi_departament AND
          codi_seu = p_codi_seu;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- esborrem el registre
  DELETE FROM departament_seu
    WHERE codi_departament = p_codi_departament AND
          codi_seu = p_codi_seu;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'DELETE', p_codi_departament || 
            '; ' || p_codi_seu, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'DELETE', p_codi_departament || 
               '; ' || p_codi_seu, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix un registre amb eixe codi de departament i de seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'DELETE', p_codi_departament || 
                '; ' || p_codi_seu, RSP);
      RETURN RSP;

-- departament_seu amb usuaris associats
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, el departament_seu te usuaris associats';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'DELETE', p_codi_departament || 
               '; ' || p_codi_seu, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'departament_seu', 'DELETE', p_codi_departament || 
               '; ' || p_codi_seu, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;




-------------------------------------------------------------------------------------------------------
--- actuacions sobre la taula categoria ---
-------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_categoria (p_nom_categoria character(30))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO categoria (codi_categoria, nom_categoria)
    VALUES ('CAT' || nextval('seq_cat'), upper(p_nom_categoria));

--registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'INSERT', p_nom_categoria, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'INSERT', p_nom_categoria, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula categoria';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'INSERT', p_nom_categoria, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_categoria (p_codi_categoria character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);
  error_fk character(1) := '';

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que la categoria existeix
  SELECT count(1) INTO comptador FROM categoria
    WHERE codi_categoria = p_codi_categoria;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- esborrem el registre
  DELETE FROM categoria
    WHERE codi_categoria = p_codi_categoria;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'DELETE', p_codi_categoria, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'DELETE', p_codi_categoria, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix una categoria amb eixe codi';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'DELETE', p_codi_categoria, RSP);
      RETURN RSP;

-- existeixen usuaris amb eixa categoria
    WHEN foreign_key_violation THEN
        RSP = 'ERROR, n-hi ha usuaris amb eixa categoria';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'DELETE', p_codi_categoria, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula categoria';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'DELETE', p_codi_categoria, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_categoria (p_codi_categoria character(10), p_nom_categoria character(30))
  RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si la categoria existeix
  SELECT count(1) INTO comptador FROM categoria
    WHERE codi_categoria = p_codi_categoria;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- actualitzem el nom de la categoria si el parametre no esta en blanc
  IF p_nom_categoria <> '' THEN
    UPDATE categoria
      SET nom_categoria = p_nom_categoria
    WHERE codi_categoria = p_codi_categoria;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'UPDATE', p_codi_categoria ||
           '; ' || p_nom_categoria, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari no te permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'UPDATE', p_codi_categoria ||
           '; ' || p_nom_categoria, RSP);
      RETURN RSP;

-- la categoria no existeix
    WHEN no_data_found THEN
      RSP = 'ERROR, codi de categoria no existeix a la taula categoria';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'UPDATE', p_codi_categoria ||
           '; ' || p_nom_categoria, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio a la taula categoria';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'UPDATE', p_codi_categoria ||
           '; ' || p_nom_categoria, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;




---------------------------------------------------------------------------------------------------------------
--- actuacions sobre la taula usuari ---
---------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_usuari (p_nom character(50), p_codi_departament character(10),
  p_codi_seu character(10), p_codi_categoria character(10), p_telefon character(9))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  error_fk character(1);
  comptador integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO usuari (codi_usuari, nom, codi_departament, codi_seu, codi_categoria, telefon)
    VALUES ('USU' || nextval('seq_usu'), upper(p_nom), p_codi_departament, p_codi_seu,
           p_codi_categoria, p_telefon);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'INSERT', p_nom || '; ' ||
           p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria || '; ' || p_telefon, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'INSERT', p_nom || '; ' ||
           p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria || '; ' || p_telefon, RSP);
      RETURN RSP;

-- errors amb les claus foranes
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, alguna clau forana -codi_categoria, codi_departament_seu o codi_departament- no existeix';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'INSERT', p_nom || '; ' ||
           p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria || '; ' || p_telefon, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula usuari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'categoria', 'INSERT', p_nom || '; ' ||
           p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria || '; ' || p_telefon, RSP);
    RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_usuari (p_codi_usuari character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que l-usuari existeix a la taula usuari
  SELECT count(1) INTO comptador FROM usuari
    WHERE codi_usuari = p_codi_usuari;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- esborrem els dispositius que te assignats l-usuari
  DELETE FROM assignat
    WHERE codi_usuari = p_codi_usuari;

-- esborrem totes les peticions de material que ha fet l-usuari
  DELETE FROM peticio_material
    WHERE codi_usuari = p_codi_usuari;

-----------------------------------------------------------------------------------
-- si l-usuari era encarregat de departament el camp codi_responsable
-- de la taula departament es posara a NULL per la restriccio ON DELETE SET NULL
-----------------------------------------------------------------------------------

-- esborrem el registre
  DELETE FROM usuari
    WHERE codi_usuari = p_codi_usuari;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'DELETE', p_codi_usuari, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'DELETE', p_codi_usuari, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix el codi usuari a la taula usuari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'DELETE', p_codi_usuari, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula usuari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'DELETE', p_codi_usuari, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_usuari (p_codi_usuari character(10), p_nom character(50),
  p_codi_departament character(10), p_codi_seu character(10), p_codi_categoria character(10),
  p_telefon character(9)) RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si l-usuari existeix
  SELECT count(1) INTO comptador FROM usuari
    WHERE codi_usuari = p_codi_usuari;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- actualitzem els camps els parametres dels quals no son buits
  IF p_nom <> '' THEN
    UPDATE usuari
      SET nom = p_nom
      WHERE codi_usuari = p_codi_usuari;
  END IF;

  IF p_codi_departament <> '' THEN
    UPDATE usuari
      SET codi_departament = p_codi_departament
      WHERE codi_usuari = p_codi_usuari;
  END IF;

  IF p_codi_seu <> '' THEN
    UPDATE usuari
      SET codi_seu = p_codi_seu
      WHERE codi_usuari = p_codi_usuari;
  END IF;

  IF p_codi_categoria <> '' THEN
    UPDATE usuari
      SET codi_categoria = p_codi_categoria
      WHERE codi_usuari = p_codi_usuari;
  END IF;

  IF p_telefon <> '' THEN
    UPDATE usuari
      SET telefon = p_telefon
      WHERE codi_usuari = p_codi_usuari;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'UPDATE', p_codi_usuari ||
           '; ' || p_nom || '; ' || p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria ||
           '; ' || p_telefon, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari no te permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'UPDATE', p_codi_usuari ||
               '; ' || p_nom || '; ' || p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria ||
               '; ' || p_telefon, RSP);
      RETURN RSP;

-- l-usuari no existeix
    WHEN no_data_found THEN
      RSP = 'ERROR, codi d-usuari no existeix a la taula usuari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'UPDATE', p_codi_usuari ||
               '; ' || p_nom || '; ' || p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria ||
               '; ' || p_telefon, RSP);
      RETURN RSP;

-- errors en claus foranes
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, alguna clau forana -codi_categoria, codi_departament_seu o codi_departament- no existeix';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'UPDATE', p_codi_usuari ||
               '; ' || p_nom || '; ' || p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria ||
               '; ' || p_telefon, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio a la taula usuari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'UPDATE', p_codi_usuari ||
               '; ' || p_nom || '; ' || p_codi_departament || '; ' || p_codi_seu || '; ' || p_codi_categoria ||
               '; ' || p_telefon, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;




-------------------------------------------------------------------------------------------------------
-- funcions sobre la taula tipus_dispositiu ---
-------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_tipus_dispositiu (p_descripcio_tipus character(30)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO tipus_dispositiu (codi_tipus, descripcio_tipus)
    VALUES ('TIP' || nextval('seq_tip'), upper(p_descripcio_tipus));

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'INSERT', p_descripcio_tipus, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'INSERT', p_descripcio_tipus, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula tipus_dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'INSERT', p_descripcio_tipus, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_tipus_dispositiu (p_codi_tipus character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que el tipus dispositiu existeix a la taula tipus_dispositiu
  SELECT count(1) INTO comptador
    FROM tipus_dispositiu
    WHERE codi_tipus = p_codi_tipus;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- esborrem el registre
  DELETE FROM tipus_dispositiu
    WHERE codi_tipus = p_codi_tipus;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'DELETE', p_codi_tipus, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'DELETE', p_codi, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix el codi usuari a la taula usuari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'DELETE', p_codi, RSP);
      RETURN RSP;

----------------------------------------------------------------------------------------------
-- error claus foranes
-- significa que n-hi ha registres a les taules dispositiu, linies_comanda, linies_pressupost
-- pla_inversions o peticio_material que es relacionen amb el registre que intentem esborrar
----------------------------------------------------------------------------------------------
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, existeixen registres a altres taules amb restriccions per clau forana';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'DELETE', p_codi, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula tipus_dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'usuari', 'DELETE', p_codi_tipus, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_tipus_dispositiu (p_codi_tipus character(10),
  p_descripcio_tipus character(30)) RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- actualitzem el camp del nom si el parametre no es buit
  IF p_descripcio_tipus <> '' THEN
    UPDATE tipus_dispositiu
      SET descripcio_tipus = p_descripcio_tipus
      WHERE codi_tipus = p_codi_tipus;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'UPDATE',
           p_codi_tipus || '; ' || p_descripcio_tipus, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari no te permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'UPDATE',
           p_codi_tipus || '; ' || p_descripcio_tipus, RSP);
      RETURN RSP;

-- el codi tipus no existeix a la taula tipus_dispositiu
    WHEN no_data_found THEN
      RSP = 'ERROR, codi tipus no existeix a la taula tipus_dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'UPDATE',
           p_codi_tipus || '; ' || p_descripcio_tipus, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio a la taula tipus_dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'tipus_dispositiu', 'UPDATE',
           p_codi_tipus || '; ' || p_descripcio_tipus, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;




-------------------------------------------------------------------------------------------------------
-- funcions sobre la taula dispositiu ---
-------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_dispositiu (p_codi_tipus character(10), p_numero_serie character(20),
  p_valor real) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari NOT IN ('administrador', 'tecnic', 'encarregat') THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO dispositiu (codi_dispositiu, codi_tipus, numero_serie, data_instal_lacio, valor)
    VALUES ('DIS' || nextval('seq_dis'), p_codi_tipus, upper(p_numero_serie), current_date, p_valor);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'INSERT', p_codi_tipus ||
           '; ' || p_numero_serie || '; ' || p_valor, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'INSERT', p_codi_tipus ||
           '; ' || p_numero_serie || '; ' || p_valor, RSP);
      RETURN RSP;

-- clau forana a tipus_dispositiu inexitent
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, la clau forana a tipus de dispositiu no existeix';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'INSERT', p_codi_tipus ||
           '; ' || p_numero_serie || '; ' || p_valor, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula tipus_dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'INSERT', p_codi_tipus ||
           '; ' || p_numero_serie || '; ' || p_valor, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION baixa_dispositiu (p_codi_dispositiu character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que el dispositiu existeix a la taula dispositiu
  SELECT count(1) INTO comptador FROM dispositiu
    WHERE codi_dispositiu = p_codi_dispositiu;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

------------------------------------------------------------------------------------
/* esborrem el registre
entendrem que si decidim esborrar un dispositiu ja no ens interesara que aparega:
ni en els inventaris de les seus
ni en el registre de moviments
ni en la taula d-assignacio a usuaris

a les taules assignat i linies_inventaris s'esborrraran els registres associats
per la restriccio ON DELETE CASCADE
a la taula moviments el que farem es deixar el codi_usuari amb valor NULL */
------------------------------------------------------------------------------------

  DELETE FROM dispositiu
    WHERE codi_dispositiu = p_codi_dispositiu;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'DELETE', p_codi_dispositiu, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'DELETE', p_codi_dispositiu, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix el codi dispositiu a la taula dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'DELETE', p_codi_dispositiu, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'DELETE', p_codi_dispositiu, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_dispositiu (p_codi_dispositiu character(10), p_codi_tipus character(10),
  p_numero_serie character(20), p_valor real) RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si el dispositiu existeix
  SELECT count(1) INTO comptador FROM dispositiu
    WHERE codi_dispositiu = p_codi_dispositiu;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- actualitzem el camps els parametres dels quals no estan en blanc
  IF p_codi_tipus <> '' THEN
    UPDATE dispositiu
      SET codi_tipus = p_codi_tipus
      WHERE codi_tipus = p_codi_tipus;
  END IF;

  IF p_numero_serie <> '' THEN
    UPDATE dispositiu
      SET numero_serie = upper(p_numero_serie)
      WHERE codi_tipus = p_codi_tipus;
  END IF;

  IF p_valor <> 0.0 THEN
    UPDATE dispositiu
      SET valor = p_valor
      WHERE codi_tipus = p_codi_tipus;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE',
           p_codi_dispositiu || '; ' || p_codi_tipus || '; ' || p_numero_serie || '; ' ||  p_valor, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari no te permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE',
           p_codi_dispositiu || '; ' || p_codi_tipus || '; ' || p_numero_serie || '; ' ||  p_valor, RSP);
      RETURN RSP;

-- el codi tipus no existeix a la taula dispositiu
    WHEN no_data_found THEN
       RSP = 'ERROR, dispositiu no existeix a la taula dispositiu';      
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE',
           p_codi_dispositiu || '; ' || p_codi_tipus || '; ' || p_numero_serie || '; ' ||  p_valor, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio del dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE',
           p_codi_dispositiu || '; ' || p_codi_tipus || '; ' || p_numero_serie || '; ' ||  p_valor, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;



-------------------------------------------------------------------------------------------------------
-- funcions sobre la taula proveidors --
-------------------------------------------------------------------------------------------------------
-- nomes seran obligatoris els camps: nif y nom per a donar d-alta un proveidor
-- si volem inserir mes informacio farem servir la funcio modificar_proveidor()
-------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION alta_proveidor (p_NIF character(9), p_nom character(30))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim el registre
  INSERT INTO proveidor (codi_proveidor, "NIF", nom)
    VALUES ('PRO' || nextval('seq_pro'), upper(p_NIF), upper(p_nom));

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'INSERT', p_NIF ||
           '; ' || p_nom, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'INSERT', p_NIF ||
           '; ' || p_nom, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula proveidors';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'INSERT', p_NIF ||
           '; ' || p_nom, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------
-- pense que no es convenient implementar una funcio per a donar de baixa un proveidor
-- Encara que no vullguerem treballar mes amb ell, tenim informacio en altres tables relacionades
-- (ej: presupostos, comandes i l'estadistica dels temps de lliurament que ens podrien ser d-utilitat
-- les quals quedarien inconsistents si esborrarem el proveidor del qual depenen 
-------------------------------------------------------------------------------------------------------------
-- el que fem doncs es afegir un camp bolean que ens diga que el proveidor no esta actiu --------------------
-------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION baixa_proveidor (p_codi_proveidor character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem que l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que el proveidor existeix
  SELECT count(1) INTO comptador FROM proveidor
    WHERE codi_proveidor = p_codi_proveidor;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

------------------------------------------------------------------------------------------------
-- esborrarem totes les linies de pressupost del proveidor, de les quals encara no tingam preu
------------------------------------------------------------------------------------------------
  DELETE FROM linies_pressupost
    WHERE codi_pressupost IN (
                             SELECT codi_pressupost
                             FROM pressupostos
                             WHERE codi_proveidor = p_codi_proveidor
                             ) AND
    preu = 0.0;

-------------------------------------------------------------------------------
-- aixi mateix si es queda algun pressupost sense cap linia l-esborrarem tambe
-------------------------------------------------------------------------------
  DELETE FROM pressupostos
    WHERE codi_pressupost NOT IN (
			         SELECT codi_pressupost
			         FROM linies_pressupost
				 ) AND
    codi_proveidor = p_codi_proveidor;


-- ara marcarem el proveidor com inactiu
  UPDATE proveidor
    SET baixa = TRUE
    WHERE codi_proveidor = p_codi_proveidor;


-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE', p_codi_proveidor, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE', p_codi_proveidor, RSP);
      RETURN RSP;

-- registre inexistent
    WHEN no_data_found THEN
      RSP = 'ERROR, no existeix el codi dispositiu a la taula dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE', p_codi_proveidor, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible donar de baixa el registre a la taula dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'dispositiu', 'UPDATE', p_codi_proveidor, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------------



CREATE OR REPLACE FUNCTION modificacio_proveidor (p_codi_proveidor character(10), p_NIF character(9),
  p_nom character(30), p_adreca character(30), p_ciutat character(30), p_provincia character(30),
  p_codi_postal character(5)) RETURNS character(100) AS $RSP$

DECLARE

  RSP character(100);
  comptador integer = 0;
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si el proveidor existeix
  SELECT count(1) INTO comptador FROM proveidor
    WHERE codi_proveidor = p_codi_proveidor;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

-- actualitzem el camps els parametres dels quals no estan en blanc
  IF p_NIF <> '' THEN
    UPDATE proveidor
      SET NIF = p_NIF
      WHERE codi_proveidor = p_codi_proveidor;
  END IF;  

  IF p_nom <> '' THEN
    UPDATE proveidor
      SET nom = p_nom
      WHERE codi_proveidor = p_codi_proveidor;
  END IF;  

  IF p_adreca <> '' THEN
    UPDATE proveidor
      SET adreca = p_adreca
      WHERE codi_proveidor = p_codi_proveidor;
  END IF;  

  IF p_ciutat <> '' THEN
    UPDATE proveidor
      SET ciutat = p_ciutat
      WHERE codi_proveidor = p_codi_proveidor;
  END IF;  
  
  IF p_provincia <> '' THEN
    UPDATE proveidor
      SET provincia = p_provincia
      WHERE codi_proveidor = p_codi_proveidor;
  END IF;
  
  IF p_codi_postal <> '' THEN
    UPDATE proveidor
      SET codi_postal = p_codi_postal
      WHERE codi_proveidor = p_codi_proveidor;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'UPDATE',
           p_codi_proveidor || '; ' || p_NIF || '; ' || p_nom || '; ' || p_adreca || '; ' ||
           p_ciutat || '; ' || p_provincia || '; ' || p_codi_postal, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari no te permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'UPDATE',
           p_codi_proveidor || '; ' || p_NIF || '; ' || p_nom || '; ' || p_adreca || '; ' ||
           p_ciutat || '; ' || p_provincia || '; ' || p_codi_postal, RSP);
      RETURN RSP;

-- no existeix el registre que intentem esborrar
    WHEN no_data_found THEN
      RSP = 'ERROR, el codi proveidor no existeis a la taula proveidor';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'UPDATE',
           p_codi_proveidor || '; ' || p_NIF || '; ' || p_nom || '; ' || p_adreca || '; ' ||
           p_ciutat || '; ' || p_provincia || '; ' || p_codi_postal, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible fer la modificacio a la taula proveidor';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'proveidor', 'UPDATE',
           p_codi_proveidor || '; ' || p_NIF || '; ' || p_nom || '; ' || p_adreca || '; ' ||
           p_ciutat || '; ' || p_provincia || '; ' || p_codi_postal, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-----------------------------------------------------------------------------------------------------------