﻿------------------------------------
-- P A Q U E T        G E S T I O --
------------------------------------


/*
Al paquet gestio estaran totes les funcions que formen la "logica de negoci" de la empresa
Les funcions aci incloses les poden executar diversos usuaris que es controlaran dintre
de cada funcio.
*/


---------------------------------------------------------------------------------
-- aquesta funcio l-executara qualsevol usuari que vullga demanar un material
-- els parametres introduits seran el codi de l-usuari que fa la peticio,
-- el tipus de material que es demana i la quantitat de unitats requerides
---------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION demanar_material (p_codi_usuari character(10), p_codi_tipus character(10),
  p_quantitat integer) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- no comprovem si l-usuari te permisos perque qualsevol usuari pot fer una peticio
  SELECT user INTO var_usuari;

-- inserim el registre
  INSERT INTO peticio_material (codi_peticio, codi_usuari, codi_tipus, quantitat, estat, data_peticio)
    VALUES ('PET' || nextval('seq_pet'), p_codi_usuari, p_codi_tipus, p_quantitat, 'D', current_date);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'INSERT',
           p_codi_usuari || '; ' || p_codi_tipus || '; ' || p_quantitat, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari o tipus de dispositiu inexistents
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, usuari y/o tipus de dispositiu inexistent';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'INSERT',
           p_codi_usuari || '; ' || p_codi_tipus || '; ' || p_quantitat, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible actualitzar la taula peticio_material';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'INSERT',
           p_codi_usuari || '; ' || p_codi_tipus || '; ' || p_quantitat, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------
-- aquesta funcio la podra executar nomes l-usuari responsable
-- com a parametres es pasaran el codi del usuari que aprova la peticio
-- i el codi de la peticio que va a aprovar
------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION aprovar_peticio (p_codi_responsable character(10), p_codi_peticio character(30))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'responsable' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si existeix la peticio
  SELECT count(1) INTO comptador FROM peticio_material
    WHERE codi_peticio = p_codi_peticio;
  IF comptador = 0 THEN
    RAISE EXCEPTION no_data_found;
  END IF;

------------------------------------------------------------------------------------
-- comprovem si el codi de usuari responsable introduit es realment el responsable
-- del departament al qual pertany l-usuari que ha fet la sol-licitud
------------------------------------------------------------------------------------
  IF p_codi_responsable NOT IN (
    SELECT d.codi_responsable
      FROM departament d, usuari u, peticio_material pm
      WHERE d.codi_departament = u.codi_departament AND
            u.codi_usuari = pm.codi_usuari AND
	    p_codi_peticio = pm.codi_peticio) THEN
    RAISE EXCEPTION data_exception;
  END IF;

-- modifiquem el camp estat a la taula posant una A de aprovat
  UPDATE peticio_material
    SET estat = 'A'
    WHERE codi_peticio = p_codi_peticio;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

-- la peticio no existeix
    WHEN no_data_found THEN
      RSP = 'ERROR, el codi de peticio no existeix a la taula peticio_material';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

-- el codi de responsable no existeix o no es responsable del mateix departament del usuari que ha fet la peticio
    WHEN data_exception THEN
      RSP = 'ERROR, codi responsable incorrecte o no pertany al mateix departament del usuari peticionari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible actualitzar la taula peticio_material';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

------------------------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------
-- aquesta funcio la podra executar nomes l-usuari responsable
-- Com a parametres es pasaran el codi del usuari que rebutja la peticio
-- i el codi de la peticio que va a rebutjar
------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION rebutjar_peticio (p_codi_responsable character(10), p_codi_peticio character(30))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'responsable' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si existeix la peticio
  SELECT count(1) INTO comptador FROM peticio_material
    WHERE codi_peticio = p_codi_peticio;
  IF comptador = 0 THEN
    RAISE EXCEPTION no_data_found;
  END IF;

-----------------------------------------------------------------------------------
-- comprovem si el codi de usuari responsable introduit es realment el responsable
-- del departament al qual pertany l-usuari que ha fet la sol-licitud
------------------------------------------------------------------------------------
  IF p_codi_responsable NOT IN (
    SELECT d.codi_responsable
      FROM departament d, usuari u, peticio_material pm
      WHERE d.codi_departament = u.codi_departament AND
            u.codi_usuari = pm.codi_usuari AND
	    p_codi_peticio = pm.codi_peticio) THEN
    RAISE EXCEPTION data_exception;
  END IF;

-- modifiquem el camp estat posant una R de rebutjat
  UPDATE peticio_material
    SET estat = 'R'
    WHERE codi_peticio = p_codi_peticio;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

-- la peticio no existeix
    WHEN no_data_found THEN
      RSP = 'ERROR, el codi de peticio no existeix a la taula peticio_material';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

-- el codi de responsable no existeix o no es responsable del mateix departament del usuari que ha fet la peticio
    WHEN data_exception THEN
      RSP = 'ERROR, codi responsable incorrecte o no pertany al mateix departament del usuari peticionari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'UPDATE', p_codi_responsable
                  || '; ' || p_codi_peticio, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------
-- aquesta funcio l-executara l-usuari subministraments i li servira per a saber sobre quins
-- tipus de material ha de demanar pressupostos i fer comandes
-- La funcio no te parametres d-entrada i torna com a eixida un llistats amb
-- totes les peticions aprovades pels responsables dels departaments
-----------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_peticions_aprovades () RETURNS SETOF peticio_material AS $resultat$

DECLARE

  resultat peticio_material%rowtype;
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'subministraments' THEN
    RAISE invalid_role_specification;
  END IF;

----------------------------------------------------------------
-- recorrerem la taula peticio_de material i seleccionarem
-- aquells registres que tenen una A en el camp estat, es a dir
-- aquelles peticions que estan aprovades pels responsables
-- del departament de l-usuari peticionari
----------------------------------------------------------------
  FOR resultat IN
  SELECT *
    FROM peticio_material
    WHERE estat = 'A'
    ORDER BY data_peticio
  LOOP
  RETURN NEXT resultat;

  END LOOP;

  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula seu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'peticio_material', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta';

END;
$resultat$LANGUAGE plpgsql;

---------------------------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------
-- aquesta funcio podra ser executada per l-usuari subministraments i creara
-- la capçelera de una peticio de pressupost
-- Nomes te un parametre d-entrada que es el codi del proveidor al qual fem
-- la peticio de pressupost  
-----------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION demanar_pressupost (p_codi_proveidor character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'subministraments' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim un nou registre a la taula pressupostos
  INSERT INTO pressupostos (codi_pressupost, codi_proveidor, data)
    VALUES ('PRE' || nextval('seq_pre'), p_codi_proveidor, current_date);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pressupostos', 'INSERT', p_codi_proveidor, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pressupostos', 'INSERT', p_codi_proveidor, RSP);
      RETURN RSP;

-- el codi de proveidor no existeix
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi proveidor incorrecte';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pressupostos', 'INSERT', p_codi_proveidor, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible crear el pressuposto';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pressupostos', 'INSERT', p_codi_proveidor, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

---------------------------------------------------------------------------------------------------------------------------


-------------------------------------------------------------------------------------------------------------------------
-- funcio per a inserir les linies de presupost que corresponen a la capçelera creada amb la funcio demanar_pressupost()
-- Els parametres d-entrada son el codi del presupost i el codi del tipus de material del qual volem
-- demanar pressupost
--------------------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION inserir_linia_pressupost (p_codi_pressupost character(10), p_codi_tipus character(10))
  RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  var_num_linia integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'subministraments' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim una nova linia a la taula
  IF (SELECT count(1)
      FROM linies_pressupost lp
      WHERE codi_pressupost = p_codi_pressupost) = 0 THEN
        INSERT INTO linies_pressupost (codi_pressupost, numero_linia, codi_tipus)
        VALUES (p_codi_pressupost, 1, p_codi_tipus);
  ELSE
    SELECT max(numero_linia) INTO var_num_linia
      FROM linies_pressupost lp
      WHERE codi_pressupost = p_codi_pressupost;
    INSERT INTO linies_pressupost (codi_pressupost, numero_linia, codi_tipus)
      VALUES (p_codi_pressupost, (var_num_linia + 1), p_codi_tipus);
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'INSERT', p_codi_pressupost
            || '; ' || p_codi_tipus, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'INSERT', p_codi_pressupost
            || '; ' || p_codi_tipus, RSP);
      RETURN RSP;

-- el codi de pressupost o el codi de tipus de material no existeix
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi proveidor o codi tipus incorrecte';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'INSERT', p_codi_pressupost
            || '; ' || p_codi_tipus, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir la linia de pressupost';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'INSERT', p_codi_pressupost
            || '; ' || p_codi_tipus, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

------------------------------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------------------------
-- funcio complementaria a les dos anteriors per tal d-actualitzar el preu del article del qual hem demanat pressupost
----------------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION actualitzar_preu_pressupost (p_codi_pressupost character(10), p_numero_linia integer,
  p_preu real) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  var_num_linia integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'subministraments' THEN
    RAISE invalid_role_specification;
  END IF;

-- actualitzem el preu del article
  IF p_preu <> 0 THEN
    UPDATE linies_pressupost
      SET preu = p_preu
      WHERE codi_pressupost = p_codi_pressupost AND
            numero_linia = p_numero_linia;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'UPDATE', p_codi_pressupost
            || '; ' || p_numero_linia || '; ' || p_preu, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'UPDATE', p_codi_pressupost
            || '; ' || p_numero_linia || '; ' || p_preu, RSP);
      RETURN RSP;

-- el codi de pressupost o el numero de linia no existeix
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi proveidor o numero de linia incorrecte';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'UPDATE', p_codi_pressupost
            || '; ' || p_numero_linia || '; ' || p_preu, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir o modificar el preu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_pressupost', 'UPDATE', p_codi_pressupost
            || '; ' || p_numero_linia || '; ' || p_preu, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------
-- funcio per a crear la capçalera d-una comanda
-- te permisos per a executarla l-usuari subministraments
-- els parametres d-entrada son el codi del proveidor al qual li fem la comanda i
-- un valor boolean que ens indicara si la comanda pertany a una ordre d-inversio
-- de ser aixi ha d-existir un registre amb els tipus d-articles que demanem a la
-- taul pla_inversions
------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION fer_comanda(p_codi_proveidor character(10), p_ordre_inversio boolean) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'subministraments' THEN
    RAISE invalid_role_specification;
  END IF;

-- inserim un nou registre a la taula comandes
  INSERT INTO comandes (codi_comanda, codi_proveidor, data, ordre_inversio)
    VALUES ('COM' || nextval('seq_com'), p_codi_proveidor, current_date, p_ordre_inversio);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'comandes', 'INSERT', p_codi_proveidor
            || '; ' || p_ordre_inversio, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'comandes', 'INSERT', p_codi_proveidor
            || '; ' || p_ordre_inversio, RSP);
      RETURN RSP;

-- el codi de proveidor no existeix
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi proveidor incorrecte';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'comandes', 'INSERT', p_codi_proveidor
            || '; ' || p_ordre_inversio, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible crear la comanda';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'comandes', 'INSERT', p_codi_proveidor
            || '; ' || p_ordre_inversio, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

---------------------------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------------------
-- funcio per afegir linies de comanda en la capçelera creada amb la funcio fer_comanda()
-- si els articles demanats pertanyen a una ordre d-inversio comprobarem que existeix un registre
-- amb el tipus d-article a la taula pla_inversions i que encara no hem superat ni les unitats
-- ni l-import reservat en aquest registre
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION inserir_linia_comanda (p_codi_comanda character(10), p_codi_tipus character(10),
  p_unitats integer, p_preu_unitari real) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  var_num_linia integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'subministraments' THEN
    RAISE invalid_role_specification;
  END IF;

---------------------------------------------------------------------------------
-- si la comanda correspona una ordre d-inversio comprobarem que esta inclosa
-- al pla d-inversions del any en curs i que encara no s-ha esgotat l-import
-- reservat per al tipus de dispositiu que es vol demanar
---------------------------------------------------------------------------------
  IF p_codi_tipus IN (SELECT codi_tipus
		      FROM pla_inversions
		      WHERE p_codi_tipus = codi_tipus AND
                            "any" = extract(year from current_date)
		      )
		      AND (p_unitats * p_preu_unitari) < (SELECT import
							  FROM pla_inversions
						  	  WHERE p_codi_tipus = codi_tipus AND
							        "any" = extract(year from current_date)
							  ) THEN
    UPDATE pla_inversions
    SET import = import - (p_unitats * p_preu_unitari)
    WHERE codi_tipus = p_codi_tipus AND
          "any" = extract(year from current_date);

  ELSE
	RAISE data_exception;
  END IF;

-- inserim una nova linia a la taula
  IF (SELECT count(1)
      FROM linies_comanda lc
      WHERE codi_comanda = p_codi_comanda) = 0 THEN
        INSERT INTO linies_comanda (codi_comanda, numero_linia, codi_tipus, unitats, preu_unitari)
        VALUES (p_codi_comanda, 1, p_codi_tipus, p_unitats, p_preu_unitari);
  ELSE
    SELECT max(numero_linia) INTO var_num_linia
      FROM linies_comanda lc
      WHERE codi_comanda = p_codi_comanda;
    INSERT INTO linies_comanda (codi_comanda, numero_linia, codi_tipus, unitats, preu_unitari)
      VALUES (p_codi_comanda, (var_num_linia + 1), p_codi_tipus, p_unitats, p_preu_unitari);
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_comanda', 'INSERT', p_codi_comanda
            || '; ' || p_codi_tipus || '; ' || p_unitats || '; ' || p_preu_unitari, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_comanda', 'INSERT', p_codi_comanda
            || '; ' || p_codi_tipus || '; ' || p_unitats || '; ' || p_preu_unitari, RSP);
      RETURN RSP;

-- tipus de dispositiu no inclos en pla d-inversions o no n-hi ha import suficient
    WHEN data_exception THEN
      RSP = 'ERROR, tipus dispositius no inclos en pla d-inversions o no n-hi ha prou import reservat';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_comanda', 'INSERT', p_codi_comanda
            || '; ' || p_codi_tipus || '; ' || p_unitats || '; ' || p_preu_unitari, RSP);
      RETURN RSP;

-- el codi de proveidor o el codi tipus no existeix
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi proveidor o codi tipus incorrecte';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_comanda', 'INSERT', p_codi_comanda
            || '; ' || p_codi_tipus || '; ' || p_unitats || '; ' || p_preu_unitari, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir la linia de comanda';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_comanda', 'INSERT', p_codi_comanda
            || '; ' || p_codi_tipus || '; ' || p_unitats || '; ' || p_preu_unitari, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

----------------------------------------------------------------------------------------------------------------------


-------------------------------------------------------------------------------------------------
-- funcio que executara l-usuari comptabilitat per a definir el pla d-inversions per a l-exercici
-- Dels articles que es demanen d-aquest pla d-inversions s-haura de fer l-amortitzacio comptable
-- que corresponga
-- Els parametres d-entrada seran l-exercici sobre el que anem a fer el pla, el tipus de
-- dispositiu a comprar, les unitats i l-import maxim que podem gastar en la compra
--------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION definir_pla_inversions (p_any integer, p_codi_tipus character(10),
  p_quantitat integer, p_import real) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  var_any_actual integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'comptabilitat' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que l-any pasat com a parametre es igual o superior a l-actual
  SELECT extract (year from current_date) INTO var_any_actual;
  IF p_any < var_any_actual THEN
    RAISE data_exception;
  END IF;

-- inserim el registre
  INSERT INTO pla_inversions (codi_pla, "any", codi_tipus, quantitat, import)
    VALUES ('PLA' || nextval('seq_pla'), p_any, p_codi_tipus, p_quantitat, p_import);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pla_inversions', 'INSERT',
           p_any || '; ' || p_codi_tipus || '; ' || p_quantitat || '; ' || p_import, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pla_inversions', 'INSERT',
           p_any || '; ' || p_codi_tipus || '; ' || p_quantitat || '; ' || p_import, RSP);
      RETURN RSP;

-- any anterior a l-actual
    WHEN data_exception THEN
      RSP = 'ERROR, no es pot modificar un pla d-un any anterior a l-actual';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pla_inversions', 'INSERT',
           p_any || '; ' || p_codi_tipus || '; ' || p_quantitat || '; ' || p_import, RSP);

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible actualitzar la taula pla_inversions';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'pla_inversions', 'INSERT',
           p_any || '; ' || p_codi_tipus || '; ' || p_quantitat || '; ' || p_import, RSP);

END;
$RSP$ LANGUAGE plpgsql;



-----------------------------------------------------------------------------------------------------
-- almenys una volta a l-any estarem obligats a fer un inventari fisic dels dispositius
-- assignats al demartament
-- Aquesta funcio creara la capçelera de l-inventari
-- L-usuari autoritzat a executar aquesta funcio es responsable
-- Com a parametres pasarem el codi d-usuari del responsable del departament que anem a inventariar i
-- el codi d-aquest mateix departament
-----------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION fer_inventari_anual (p_codi_responsable character(10),
  p_codi_departament character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'responsable' THEN
    RAISE invalid_role_specification;
  END IF;

----------------------------------------------------------------------------------
-- comprovem si el codi d-usuari responsable introduit es realment el responsable
-- del departament del qual anem a fer l-inventari
----------------------------------------------------------------------------------
  IF p_codi_responsable NOT IN (
    			       SELECT codi_responsable
      			       FROM departament
      			       WHERE codi_departament = p_codi_departament
                               ) THEN
    RAISE EXCEPTION invalid_role_specification;
  END IF;

-- inserim un nou registre a la taula inventari
  INSERT INTO inventari (codi_inventari, codi_departament, data)
    VALUES ('INV' || nextval('seq_inv'), p_codi_departament, current_date);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'inventari', 'INSERT',
            p_codi_responsable || '; ' || p_codi_departament, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos o no es el responsable del depament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'inventari', 'INSERT',
                p_codi_responsable || '; ' || p_codi_departament, RSP);
      RETURN RSP;

-- codi usuari responsable o codi seu erroni
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi departament incorrectes';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'inventari', 'INSERT',
                p_codi_responsable || '; ' || p_codi_departament, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible crear l-inventari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'inventari', 'INSERT',
                p_codi_responsable || '; ' || p_codi_departament, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

---------------------------------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------
-- como complement a la funcio anterior, amb aquesta anirem inserint linies d-inventari en el
-- nostre inventari anual
-----------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION inserir_linia_inventari (p_codi_inventari character(10),
  p_codi_dispositiu character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  var_num_linia integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'responsable' THEN
    RAISE invalid_role_specification;
  END IF;

---------------------------------------------------------------------------------
-- comprovarem si el codi de dispositiu que estem inventariant esta assignat a
-- un usuari que siga del departament que estem inventariant
---------------------------------------------------------------------------------
  IF p_codi_dispositiu NOT IN (
    SELECT dis.codi_dispositiu
    FROM inventari inv, departament dep, usuari usu, assignat ass, dispositiu dis
    WHERE p_codi_inventari = inv.codi_inventari AND
          inv.codi_departament = dep.codi_departament AND
          dep.codi_departament = usu.codi_departament AND
          usu.codi_usuari = ass.codi_usuari AND
          ass.codi_dispositiu = dis.codi_dispositiu)
  THEN
    RAISE no_data_found;
  END IF; 

-- inserim una nova linia a la taula linies_inventari
  INSERT INTO linies_inventari (codi_inventari, codi_dispositiu)
    VALUES (p_codi_inventari, p_codi_dispositiu);

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_inventari', 'INSERT',
            p_codi_inventari || '; ' || p_codi_dispositiu, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_inventari', 'INSERT',
            p_codi_inventari || '; ' || p_codi_dispositiu, RSP);
      RETURN RSP;

-- codi dispositiu no pertany a un usuari del departament que estem inventariant
    WHEN no_data_found THEN
      RSP = 'ERROR, codi dispositiu no pertany al departament';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_inventari', 'INSERT',
            p_codi_inventari || '; ' || p_codi_dispositiu, RSP);
      RETURN RSP;

-- codi dispositiu repetit per aquest inventari
    WHEN unique_violation THEN
      RSP = 'ERROR, codi dispositiu ja inventariat';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_inventari', 'INSERT',
            p_codi_inventari || '; ' || p_codi_dispositiu, RSP);
      RETURN RSP;

-- codi dispositiu inexistent o erroni
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, codi dispositiu inexistent o codi erroni';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_inventari', 'INSERT',
            p_codi_inventari || '; ' || p_codi_dispositiu, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir la linia de inventari';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'linies_inventari', 'INSERT',
            p_codi_inventari || '; ' || p_codi_dispositiu, RSP);
      RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;



------------------------------------------------------------------------------------------------------
-- entenc que fer l-amortitzacio comptable queda fora de l-abast d'aquest projecte.
-- El que fara doncs aquesta funcio es, donat un any comptable, treure una relacio de tots
-- els articles rebuts durant l-any; la compra dels quals correspon a un pla 
-- d-inversions
-- Aquesta funcio podra ser executada per l-usuari comptabilitat i li pasarem com
-- a paremetre d-entrada l-any del que volem extreure la informacio i ens donara
-- com a exida tots els articles rebuts en eixe any que corresponen al pla d-inversions
-----------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION controlar_amortitzacio (p_any integer) RETURNS setof amortitzacio
  AS $resultat$

DECLARE

  resultat amortitzacio%rowtype;
  RSP character(100);
  var_usuari character(30);
  var_any integer;
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'comptabilitat' THEN
    RAISE invalid_role_specification;
  END IF;

--------------------------------------------------------------------------
-- extraurem tots els registres de la taula amortitzacio que corresponen
-- a l-any pasat com a parametre
--------------------------------------------------------------------------
  FOR resultat IN
  SELECT *
    FROM amortitzacio
    WHERE extract(year from data_recepcio) = p_any
    ORDER BY codi_tipus, data_recepcio
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'amortitzacio', 'SELECT', p_any, RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'amortitzacio', 'SELECT', p_any, RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- any incorrecte o sense registres
    WHEN no_data_found THEN
      RSP = 'ERROR, any incorrecte o sense pla d-inversions definit';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'amortitzacio', 'SELECT', p_any, RSP);
    RAISE NOTICE 'Any incorrecte o sense pla d-inversions definit';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible treure el llistat d-amortitzacions';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'amortitzacio', 'SELECT', p_any, RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta';

END;
$resultat$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------
-- aquesta funcio la podra executar l-usuari tecnic
-- inserira els moviments de dispositius a la taula moviments i a mes a mes
-- actualitzara la taula assignat
-- Els parametres d-entrada seran el tipus de moviment (assignacio, reassignacio, etc.),
-- el codi del dispositiu que es mou, el codi d-usuari origen (optatiu perque podria tractar-se
-- d-un dispositiu que no estaguera previament assignat a ningun usuari) i codi d-usuari desti
-- (optatiu perque es podria voler desassignar el dispositiu de qualsevol usuari)
-------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION moviment_dispositiu (p_tipus character(30), p_codi_dispositiu character(10),
  p_codi_usuari_origen character(10), p_codi_usuari_desti character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer := 0;
  var_codi_usuari_origen character(10) := null;
  var_codi_usuari_desti character(10) := null;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'tecnic' THEN
    RAISE invalid_role_specification;
  END IF;

--------------------------------------------------------------------------------
-- si ens han pasat l-usuari origen com a parametre el guardem en una variable
-- si no aquesta variable valdra NULL
--------------------------------------------------------------------------------
  IF p_codi_usuari_origen <> '' THEN
    var_codi_usuari_origen := p_codi_usuari_origen;

------------------------------------------------------------------------
-- si ens han pasat un codi d-usuari origen
-- comprovem que existeix l-assignacio previa del dispositiu a aquest
------------------------------------------------------------------------
    SELECT COUNT(1) INTO comptador
      FROM assignat
      WHERE codi_usuari = p_codi_usuari_origen AND
            codi_dispositiu = p_codi_dispositiu;
    IF comptador > 0 THEN
      DELETE FROM assignat
        WHERE codi_usuari = p_codi_usuari_origen AND
              codi_dispositiu = p_codi_dispositiu;
    ELSE
      RAISE no_data_found;
    END IF;

  END IF;

-- inserim l-assignacio del dispositiu a l-usuari desti (si es que es pasa com a parametre)
  IF p_codi_usuari_desti <> '' THEN
    var_codi_usuari_desti := p_codi_usuari_desti;
    INSERT INTO assignat (codi_usuari, codi_dispositiu, data)
      VALUES (p_codi_usuari_desti, p_codi_dispositiu, current_date);
  END IF;

-- inserim el registre a la taula moviments
  INSERT INTO moviments (codi_moviment, tipus, codi_dispositiu, codi_usuari_origen, codi_usuari_desti, data)
    VALUES ('MOV' || nextval('seq_mov'), upper(p_tipus), p_codi_dispositiu, var_codi_usuari_origen,
            var_codi_usuari_desti, current_date);

-----------------------------------------------------------------------------------------
-- si no ha fallat l-insert es perque les dades de tipus de dispositiu, usuari origen i
-- usuari desti son correctes. Aleshores registrem el log
-----------------------------------------------------------------------------------------
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'moviments i assignat', 'INSERT', p_tipus
            || '; ' || p_codi_dispositiu || '; ' || p_codi_usuari_origen || '; ' || p_codi_usuari_desti, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'moviments i assignat', 'INSERT', p_tipus
            || '; ' || p_codi_dispositiu || '; ' || p_codi_usuari_origen || '; ' || p_codi_usuari_desti, RSP);
      RETURN RSP;

-- ja existeix una assignacio del dispositiu a l-usuari desti en la taula assignat
    WHEN unique_violation THEN
      RSP = 'ERROR, l-usuari desti ja te assignat el dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'moviments i assignat', 'INSERT', p_tipus
            || '; ' || p_codi_dispositiu || '; ' || p_codi_usuari_origen || '; ' || p_codi_usuari_desti, RSP);
      RETURN RSP;

-- no existeix l-assignacio del dispositiu a l-usuari origen en la taula assignat
    WHEN no_data_found THEN
      RSP = 'ERROR, l-usuari desti ja te assignat el dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'moviments i assignat', 'INSERT', p_tipus
            || '; ' || p_codi_dispositiu || '; ' || p_codi_usuari_origen || '; ' || p_codi_usuari_desti, RSP);
      RETURN RSP;

----------------------------------------------------------------------
-- errors amb les claus foranes
-- pot ser per:
--   codi_dispositiu, codi_usuari_origen o codi_usuari_desti erronis
-----------------------------------------------------------------------
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, alguna clau forana -codi_categoria, codi_departament_seu o codi_departament- no existeix';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'moviments i assignat', 'INSERT', p_tipus
            || '; ' || p_codi_dispositiu || '; ' || p_codi_usuari_origen || '; ' || p_codi_usuari_desti, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a les taules moviments i assignat';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'moviments i assignat', 'INSERT', p_tipus
            || '; ' || p_codi_dispositiu || '; ' || p_codi_usuari_origen || '; ' || p_codi_usuari_desti, RSP);
    RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------------------
-- aquesta funcio l-executara l-usuari tecnic i inserira un registre a la taula avaries
-- marcant tambe amb true el camp avariat del registre del dispositiu
-- Els parametres d-entrada seran el codi del dispositiu avariat i la descripcio
-- de l-avaria
-----------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION registrar_avaria (p_codi_dispositiu character(10),
  p_descripcio_avaria text) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'tecnic' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que no existeix una avaria sense reparacio per al mateix dispositiu
  SELECT COUNT(1) INTO comptador
    FROM avaries
    WHERE codi_dispositiu = p_codi_dispositiu AND
          data_reparacio is null;
    IF comptador > 0 THEN
      RAISE data_exception;
    END IF;

-- inserim el registre
  INSERT INTO avaries (id_avaria, codi_dispositiu, data_avaria, descripcio_avaria)
    VALUES ('AVA' || nextval('seq_ava'), p_codi_dispositiu, current_date, p_descripcio_avaria);

------------------------------------------------------------------------------------
-- si no ha fallat l-insercio del registre es perque totes les dades son correctes,
-- aleshores posem el camp avariat del dispositiu a TRUE
------------------------------------------------------------------------------------
  UPDATE dispositiu
    SET avariat = TRUE
    WHERE codi_dispositiu = p_codi_dispositiu;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'INSERT', p_codi_dispositiu
            || '; ' || p_descripcio_avaria, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'INSERT', p_codi_dispositiu
            || '; ' || p_descripcio_avaria, RSP);
      RETURN RSP;

-- avaria ja registrada (es dir, existeix un registre del mateix dispositiu sense data de reparacio)
    WHEN data_exception THEN
      RSP = 'ERROR, ja existeix una avaria registrada per al dispositiu';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'INSERT', p_codi_dispositiu
            || '; ' || p_descripcio_avaria, RSP);
      RETURN RSP;

-- no existeix el dispositiu pasat com a parametre
    WHEN foreign_key_violation THEN
      RSP = 'ERROR, no existeix el codi de dispositiu pasat com a parametre';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'INSERT', p_codi_dispositiu
            || '; ' || p_descripcio_avaria, RSP);
      RETURN RSP;

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible inserir el registre a la taula avaries';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'INSERT', p_codi_dispositiu
            || '; ' || p_descripcio_avaria, RSP);
    RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------
-- els permisos d-execucio d-aquesta funcio els tindra l-usuari tecnic
-- El parametre d-entrada sera el codi d-avaria
-- Posara la data d-avui en el camp data_reparacio de la taula avaries i
-- posara a false el camp avariat de la taula dispositiu
----------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION registrar_reparacio (p_id_avaria character(10)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer := 0;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'tecnic' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que existeix l-avaria
  SELECT COUNT(1) INTO comptador
    FROM avaries
    WHERE id_avaria = p_id_avaria;
    IF comptador = 0 THEN
      RAISE data_exception;
    END IF;

-- modifiquem el registre a la taula avaries posant la data d-avui al camp data_reparacio
  UPDATE avaries
    SET data_reparacio = current_date
    WHERE id_avaria = p_id_avaria;

-- actualitzem el camp avariat del dispositiu posant a false el camp avariat
  UPDATE dispositiu
    SET avariat = FALSE
    WHERE codi_dispositiu IN (
                             SELECT codi_dispositiu
                             FROM avaries
                             WHERE id_avaria = p_id_avaria
                             );


-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'UPDATE', p_id_avaria, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'UPDATE', p_id_avaria, RSP);
      RETURN RSP;

-- no existeix el registre amb el codi d-avaria pasat com a parametre
    WHEN data_exception THEN
      RSP = 'ERROR, codi d-avaria incorrecte';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'UPDATE', p_id_avaria, RSP);
      RETURN RSP;


-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible actualitzar l-avaria';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'avaries', 'UPDATE', p_id_avaria, RSP);
    RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------
-- aquesta funcio la podran executar tant l-usuari tecnic com l-usuari encarregat (de la recepcio)
-- Els parametres d-entrada seran el codi de la comanda, el numero de linea de la comanda i
-- el numero de serie del dispositiu recepcionat
-- Inserira un registre a la taula recepcio_material i un altre a la taula dispositiu
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION recepcionar_dispositiu (p_codi_comanda character(10), p_numero_linia integer,
  p_numero_serie character(20)) RETURNS character(100) AS $RSP$

DECLARE
  RSP character(100);
  var_usuari character(30);
  comptador integer := 0;
  var_valor real;
  var_codi_tipus character(10);
  var_data_comanda date;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'tecnic' AND var_usuari <> 'encarregat' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem que existeix la comanda i el numero de linia
  SELECT COUNT(1) INTO comptador
    FROM linies_comanda
    WHERE codi_comanda = p_codi_comanda AND
          numero_linia = p_numero_linia;
  IF comptador = 0 THEN
    RAISE no_data_found;
  END IF;

--------------------------------------------------------------------------------------
-- ens guardem la data de la comanda en una variable que ens servira per a calcular
-- el temps que ha tardat el proveidor en servir l-article demanat
--------------------------------------------------------------------------------------
  SELECT c.data INTO var_data_comanda
  FROM comandes c, linies_comanda lc
  WHERE c.codi_comanda = lc.codi_comanda AND
        lc.codi_comanda = p_codi_comanda AND
        lc.numero_linia = p_numero_linia;

-- si el nombre d-unitats rebudes encara es menor que les demanades...
  IF (SELECT COUNT(1)
    FROM recepcio_material
    WHERE codi_comanda = p_codi_comanda AND
          numero_linia = p_numero_linia) < (
				           SELECT unitats
                                           FROM linies_comanda
                                           WHERE codi_comanda = p_codi_comanda AND
                                                 numero_linia = p_numero_linia
                                           ) THEN

-- aleshores registrarem l-entrada del material calculant el temps de lliurament del article
-- (temps transcorregut des de la data de comanda fins la data de recepcio)
  INSERT INTO recepcio_material (id_recepcio, codi_comanda, numero_linia, numero_serie, data_recepcio,
              temps_lliurament)
    VALUES ('REC' || nextval('seq_rec'), p_codi_comanda, p_numero_linia, upper(p_numero_serie),
           current_date, (var_data_comanda - current_date));
  ELSE
    RAISE data_exception;
  END IF;

------------------------------------------------------------------------------------------------
-- si el proces de recepcio ha sigut correcte inserirem un registre a la taula dispositiu amb
-- les dades que obtenim de la linea de comanda i de la recepcio del material
-- Per a fer-ho farem una crida a la funcio alta_dispositiu() passant com a parametres el tipus
-- de dispositiu, el numero de serie i el preu de compra com a valor del dispositiu
-------------------------------------------------------------------------------------------------
  SELECT preu_unitari, codi_tipus INTO var_valor, var_codi_tipus
  FROM linies_comanda
  WHERE codi_comanda = p_codi_comanda AND
        numero_linia = p_numero_linia;

  SELECT alta_dispositiu (var_codi_tipus, p_numero_serie, var_valor) INTO RSP;


------------------------------------------------------------------------------------------
-- i si correspon a una ordre d-inversio actualitzarem la taula amortitzacio per a
-- que el departament de comptabilitat (fora de l-abast d-aquest treball) faça
-- l-amortitzacio compable que corresponga
------------------------------------------------------------------------------------------
  INSERT INTO amortitzacio(id_amortitzacio, codi_tipus, valor_comptable, data_recepcio)
  VALUES ('AMO' || nextval('seq_amo'), var_codi_tipus, var_valor, current_date);


-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'recepcio_material i dispositiu',
            'INSERT', p_codi_comanda || '; ' || p_numero_linia || '; ' || p_numero_serie, RSP);
  RETURN RSP;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'recepcio_material i dispositiu',
            'INSERT', p_codi_comanda || '; ' || p_numero_linia || '; ' || p_numero_serie, RSP);
      RETURN RSP;

-- no existeix la comanda i/o el numero de linia
    WHEN no_data_found THEN
      RSP = 'ERROR, la comanda i/o el numero de linia son incorrectes';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'recepcio_material i dispositiu',
            'INSERT', p_codi_comanda || '; ' || p_numero_linia || '; ' || p_numero_serie, RSP);
      RETURN RSP;

-- ja s-han rebut totes les unitats demanades
    WHEN data_exception THEN
      RSP = 'ERROR, ja s-han rebut totes les unitats de la comanda i linia';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'recepcio_material i dispositiu',
            'INSERT', p_codi_comanda || '; ' || p_numero_linia || '; ' || p_numero_serie, RSP);
      RETURN RSP;

---------------------------------------------------------------------------------------------------------
-- no cal capturar l-error foreign_key_violation del INSERT perque ja em comprobat que el codi_comanda
-- i el numero_linia existeixen a la taula linies_comanda
---------------------------------------------------------------------------------------------------------

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible registrar la recepcio de material';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'recepcio_material i dispositiu',
            'INSERT', p_codi_comanda || '; ' || p_numero_linia || '; ' || p_numero_serie, RSP);
    RETURN RSP;

END;
$RSP$ LANGUAGE plpgsql;

--------------------------------------------------------------------------------------------------------------------
