﻿-------------------------------------------
-- P A Q U E T       E S T A D I S T I C --
-------------------------------------------

------------------------------------------------------------------------------
-- en aquest paquet estaran totes les funcions per a recuperar l-informacio
-- estadistica que em anat mantenint amb els procediments cridats pels
-- disparadors associats a les taules assignat avaries, recepcio_material
-- i linies_comanda
-- tambe n-hi haura una funcio per recuperar els registres de la taula logs
------------------------------------------------------------------------------
-- nomes l-usuari administrador podra executar les funcions d-aquest paquet
------------------------------------------------------------------------------

-------------------------------------------------------------------------------------


--------------------------------------------------------------------
-- est1 - Donat un any concret, nombre total de dispositius comprats
--------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est1(p_any integer) RETURNS integer AS $resultat$

DECLARE

  resultat integer;
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprobem si p_any existeix
  SELECT COUNT(1) INTO comptador
  FROM "EST1"
  WHERE "any" = p_any;

  IF comptador > 0
  THEN
    SELECT nombre_disps INTO resultat
    FROM "EST1"
    WHERE "any" = p_any;
  ELSE
    RAISE no_data_found;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST1', 'SELECT', '', RSP);

  RETURN resultat;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST1', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- any no trobat a la taula "EST1"
    WHEN no_data_found THEN
      RSP = 'ERROR, any sense informacio';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST1', 'SELECT', '', RSP);
    RAISE NOTICE 'No hi ha informacio per a l-any';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 1';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST1', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est1';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------
-- est2 - Percentatge d-ordinadors portatils que no han sofert cap avaria des de la data d-assignacio
--            a un usuari (entendrem que es refereix a la darrera data d-assignacio si es que el
--            dispositiu s-ha assignat a mes d-un usuari
------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est2() RETURNS real AS $resultat$

DECLARE

  resultat real;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem el valor de l-unic registre de "EST2"
  SELECT percentatge INTO resultat
  FROM "EST2"
  LIMIT 1;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST2', 'SELECT', '', RSP);

  RETURN resultat;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST2', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 2';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST2', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est2';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

--------------------------------------------------------------------------------------------
-- est3 - Donat un any, import de les comandes realitzades associades a una ordre d-inversio
--------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est3(p_any integer) RETURNS real AS $resultat$

DECLARE

  resultat real;
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprobem si p_any existeix
  SELECT COUNT(1) INTO comptador
  FROM "EST3"
  WHERE "any" = p_any;

  IF comptador > 0
  THEN
    SELECT import INTO resultat
    FROM "EST3"
    WHERE "any" = p_any;
  ELSE
    RAISE no_data_found;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST3', 'SELECT', '', RSP);

  RETURN resultat;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST3', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- usuari sense permisos
    WHEN no_data_found THEN
      RSP = 'ERROR, p_any inexistent';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST3', 'SELECT', '', RSP);
    RAISE NOTICE 'L-any pasat com a parametre no existeix a la taula est3';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 3';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST3', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est3';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------
-- est4 - Proveidor que te un periode d-entrega mes curt (considerarem que es refereix al proveidor amb el 
--          periode MITJA de entrega mes curt)
-----------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est4() RETURNS SETOF "EST4" AS $resultat$

DECLARE

  resultat "EST4"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST4"
  FOR resultat IN SELECT * FROM "EST4"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST4', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST4', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 4';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST4', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est4';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------
-- est5 - Donat un departament, nombre de dispositius en us
-----------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est5(p_codi_departament character(10)) RETURNS SETOF "EST5" AS $resultat$

DECLARE

  resultat "EST5"%rowtype;
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprovem si el departament existeix
  SELECT COUNT(1) INTO comptador
  FROM "EST5"
  WHERE codi_departament = p_codi_departament;

  IF comptador > 0
  THEN
    FOR resultat IN
    SELECT *
    FROM "EST5"
    WHERE codi_departament = p_codi_departament
    ORDER BY codi_departament, codi_tipus
    LOOP
    RETURN NEXT resultat;
    END LOOP;
  ELSE
    RAISE no_data_found;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST5', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST5', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- usuari sense permisos
    WHEN no_data_found THEN
      RSP = 'ERROR, departament incorrecte o sense dades estadistiques';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST5', 'SELECT', '', RSP);
    RAISE NOTICE 'Departament incorrecte o sense dades estadistiques';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 5';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST5', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est5';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------
-- est6 - Per a cada departament, valor del seu inventari
---------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est6() RETURNS SETOF "EST6" AS $resultat$

DECLARE

  resultat "EST6"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST6"
  FOR resultat IN SELECT * FROM "EST6"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST6', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST6', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 6';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST6', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est6';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- est8 - Departament que en un any donat, efectua mes moviments de reassignacio
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est8(p_any integer) RETURNS SETOF "EST8" AS $resultat$

DECLARE

  resultat "EST8"%rowtype;
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprobem que p_any existeix
  SELECT COUNT(1) INTO comptador
  FROM "EST8"
  WHERE "any" = p_any;

  IF comptador > 0
  THEN

-- retornem totes les files de la taula "EST8"
  FOR resultat IN SELECT * FROM "EST8"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

  ELSE
    RAISE no_data_found;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST8', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST8', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- p_any no existent a la taula est8
    WHEN no_data_found THEN
      RSP = 'ERROR, l-any no existeix a la taula';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST8', 'SELECT', '', RSP);
    RAISE NOTICE 'L-any no existeix a la taula est8';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 8';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST8', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est8';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

--------------------------------------------------------------------
-- est9 - Percentatge d-usuaris que no tenen cap dispositiu assignat
--------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est9() RETURNS SETOF "EST9" AS $resultat$

DECLARE

  resultat "EST9"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST9"
  FOR resultat IN SELECT * FROM "EST9"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST9', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST9', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 9';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST9', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est9';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------
-- est10 - Per a cada tipus de dispositiu, nombre d-unitats que estan al magatzem sense assignar a cap usuari
-------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est10() RETURNS SETOF "EST10" AS $resultat$

DECLARE

  resultat "EST10"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST10"
  FOR resultat IN SELECT * FROM "EST10"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST10', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST10', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 10';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST10', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est10';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------
--est11 -  Departament que te mes tipus diferents de dispositius y quants tipus en te
-------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est11() RETURNS SETOF "EST11" AS $resultat$

DECLARE

  resultat "EST11"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST11"
  FOR resultat IN SELECT * FROM "EST11"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST11', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST11', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 11';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST11', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est11';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------
-- est12 - Nombre d-usuaris que tenen mes de cinq dispositius assignats
-----------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est12() RETURNS SETOF "EST12" AS $resultat$

DECLARE

  resultat "EST12"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST12"
  FOR resultat IN SELECT * FROM "EST12"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST12', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST12', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 12';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST12', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est12';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------
-- est13 - Nombre d-avaries en un any donat, a tota l-empresa
-------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est13(p_any integer) RETURNS integer AS $resultat$

DECLARE

  resultat integer;
  RSP character(100);
  var_usuari character(30);
  comptador integer;

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- comprobem que p_any existeix
  SELECT COUNT(1) INTO comptador
  FROM "EST13"
  WHERE "any" = p_any;

  IF comptador > 0
  THEN

-- retornem el valor per a p_any
  SELECT nombre_avaries INTO resultat
  FROM "EST13"
  WHERE "any" = p_any;

  ELSE
    RAISE no_data_found;
  END IF;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST13', 'SELECT', '', RSP);

  RETURN resultat;

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST13', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- any no trobat
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, l-any no existeix a la taula';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST13', 'SELECT', '', RSP);
    RAISE NOTICE 'L-any no existix a la taula est13';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 13';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST13', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est13';

END;
$resultat$ LANGUAGE plpgsql;

-------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------
-- est14 - 10 usuaris amb mes avaries registrades sobre els seus dispositius
----------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_est14() RETURNS SETOF "EST14" AS $resultat$

DECLARE

  resultat "EST14"%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST14"
  FOR resultat IN SELECT * FROM "EST14"
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST14', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST14', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible tornar el resultat de l-estadistica 14';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'EST14', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre est14';

END;
$resultat$ LANGUAGE plpgsql;

---------------------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------
-- recuperacio de tots els logs registrats al sistema
-----------------------------------------------------

CREATE OR REPLACE FUNCTION consultar_logs() RETURNS SETOF logs AS $resultat$

DECLARE

  resultat logs%rowtype;
  RSP character(100);
  var_usuari character(30);

BEGIN

-- comprovem si l-usuari te permisos
  SELECT user INTO var_usuari;
  IF var_usuari <> 'administrador' THEN
    RAISE invalid_role_specification;
  END IF;

-- retornem totes les files de la taula "EST14"
  FOR resultat IN 
    SELECT *
    FROM logs
    ORDER BY id_log
  LOOP
  RETURN NEXT resultat;
  END LOOP;

-- registrem el log
  RSP = 'OK';
  INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
    VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'logs', 'SELECT', '', RSP);

  EXCEPTION

-- usuari sense permisos
    WHEN invalid_role_specification THEN
      RSP = 'ERROR, usuari sense permisos';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'logs', 'SELECT', '', RSP);
    RAISE NOTICE 'L-usuari no te permisos per executar aquesta consulta';

-- altres errors
    WHEN OTHERS THEN
      RSP = 'ERROR, no ha sigut possible colsultar els logs';
      INSERT INTO logs (id_log, data_exec, usuari, taula, procediment, params_entrada, params_exida)
        VALUES ('LOG' || nextval('seq_log'), current_date, var_usuari, 'logs', 'SELECT', '', RSP);
    RAISE NOTICE 'No ha sigut possible executar la consulta sobre la taula logs';

END;
$resultat$ LANGUAGE plpgsql;

---------------------------------------------------------------------------------------------------------------------------
